//--------------------------------------------------------------------------
//
// Copyright 2008, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international
// treaty provisions. Cypress hereby grants to licensee a personal,
// non-exclusive, non-transferable license to copy, use, modify, create
// derivative works of, and compile the Cypress Source Code and derivative
// works for the sole purpose of creating custom software in support of
// licensee product to be used only in conjunction with a Cypress integrated
// circuit as specified in the applicable agreement. Any reproduction,
// modification, translation, compilation, or representation of this
// software except as specified above is prohibited without the express
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED,
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress' product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------
/*
*****************************************************************************
*****************************************************************************
  FILENAME: main.c
   Version: 1.0, Updated on 28 November 2008

  DESCRIPTION: Main file of the Example_PRS_PWM Project.

-----------------------------------------------------------------------------
  Copyright (c) Cypress MicroSystems 2000-2003. All Rights Reserved.
*****************************************************************************
*****************************************************************************
 
Project Objective
    To demonstrate the operation and the difference between PRS and PWM user modules of PSoC device.  

Overview
    This example demonstrates the use of the PRS user module as a pseudo random
    Pulse Width Modulator.  PRS8 and PWM8 user modules are used.  The outputs are brought to 
    P2.0 and P2.1 respectively.  RC filters are used to filter the digital output into DC
    For the same pulse width setting, the output from both the PRS and PWM are same. Two switches
    are connected to P0[0] and P0[1].  The pulse width is increased or decreased using these 
    switches.
    
    The following changes were made to the default settings in the Device Editor:
 
    Select User Modules
        o Select PWM8 from the PWMs category and rename it to PWM.
		o Select PRS8 from the Random Seq category and rename it to PRS.

    Place User Modules
        1. Place PRS8 in DBB00
        2. Place PWM8 in DBB01

Project Settings

Global Resources 
    CPU_Clock      = SysClk/8          CPU clock set to 3 MHz
    VC1 Source	   = 16	               Divide 24 Mhz system clock by 16, so that clock to PRS and PWM becomes 1.5Mhz.

User Module Parameters

   PWM:
   Clock              VC1                     VC1 is the clock to the PWM
   Enable             High                    A high input enables continuous count.
   CompareOut         Row_0_Output_1          The Output of the PWM is connected to P2[1] through Row_0_Output_1
   TerminalCountOut   None                    Not used
   Period	          255	                  Set the pulse period to 255 .
   PulseWidth	      127	                  Set the initial pulse width to 127 for 50 % duty cycle
   CompareType        Less Than Or Equal	  
   InterruptType	  Terminal Count	      Default value.  Not used in project
   ClockSync	      Sync to SysClk	      As VC1 is derived from SysClk, the clock sync is set to Sync To SysClk
   InvertEnable	      Normal	              When "Normal" is selected for InvertEnable , the enable input is active-high
   
   PRS:
   Clock	          VC1	                  VC1 is the clock to PRS
   OutputBitStream	  None	                  Output bit stream is not used
   CompareOut	      Row_0_Output_0	      The PRS PWM output is connected to P2[0] through Row_0_Output_1
   CompareType	      Less Than Or Equal	  
   ClockSync	      Sync to SysClk	      As VC1 is derived from SysClk, the clock sync is set to Sync To SysClk
   

    Note : For further information on the user module parameters, refer the User Module Data Sheet

Inputs
    Pin	            Drive           Description
    -----------------------------------------
    P0[0]		    PullDown        UP Key
	P0[1]		    PullDown        DOWN Key

Output

    Pin	    Select	            Drive           Description
    --------------------------------------------------------------
    P2[0]	GlobalOutEven_0	    Strong          PRS output
	P2[1]   GlobalOutEven_1     Strong          PWM output

Circuit Connections using CY3210 - PSoC Eval1 board
    The project can be tested using the CY3210 – PsoC Eval1 board. Following connections may be made 
    on the CY3210 board. 
    
    	Use the bred board area of the CY3210 to assemble the two RC filters.
    	Connect P2 [0] from J7 to one RC filter.
    	Connect P2[1] from J7 to the other RC filter
    	Place two push button switches on the bred board.  Connect one end of the switches to VCC
    	Connect the other end of the switches to P0[0] and P0[1] on J6

Operation
    Upon program execution all hardware settings from the device configuration are loaded 
    into the device and main.c is executed.  Both the PRS and PWM operate from VC1, which is
    configured to 1.5MHz.  Initially the duty cycle of both the PRS and PWM are set to 50%.
    Pressing the UP key on P2[0] increases the pulse width by 5 counts (approx. 2%).  Pressing
    DOWN key on P2[1] decreses the pulse width by 5 counts (approx 2%)

    */
//-----------------------------------------------------------------------------
//  Include Files
//-----------------------------------------------------------------------------
   #include <m8c.h>        // part specific constants and macros
   #include "PSoCAPI.h"    // PSoC API definitions for all User Modules

//----------------------------------------------------------------------------
// Defining  bPOLY and bSEED as 0xB8 and 0x7F , where bPOLY is used to set polynomial register 
//  and bSEED is used to set SEED register.
//-----------------------------------------------------------------------------
   #define bPOLY 0xB8 	 	// Modular Polynomial = [8,6,5,4] 
   #define bSEED 0x7F 		// Seed value (127 for 50% dutycycle)
//----------------------------------------------------------------------------
//  Global Symbols
//-----------------------------------------------------------------------------

// Variable to store the Compare Value for the PWM and PRS_PWM
BYTE CompareValue;

void main(void)
{   

    // Enable Global Interrupts.  However, this is reduntant for this project
    M8C_EnableGInt;
    
    // load the PRS polynomial 
	PRS_WritePolynomial(bPOLY);   
	
	// load the PRS seed 
	PRS_WriteSeed(bSEED);
	
	// start the PRS16 
	PRS_Start(); 

    // Enable pull down resistors on P0[0] and P0[1]
    PRT0DR &=~0x03;

    // Start the PWM
    PWM_Start();
    
    // Initialize Compare Value to 127.  This sets the duty cycle to 50%
    CompareValue = 127;
    
    // Set the Duty cycle of both the PWM and PRS_PWM
    PWM_WritePulseWidth(CompareValue);
    PRS_SEED_REG = CompareValue;
    
    // infinite loop
    while(1)
    {  
        // Check if the Up key is pressed
        if(PRT0DR & 0x01)
        {  
            // Increment CompareValue by 5
            CompareValue=CompareValue+5;
            
            // Update pulse width of PWM and PRS_PWM
            PWM_WritePulseWidth(CompareValue);
            PRS_SEED_REG = CompareValue;
            
            // Wait till the key is released
            while(PRT0DR & 0x01);
        }

        // Check if the Down key is pressed
        if(PRT0DR & 0x02)
        {  
            // Decrement the CompareValue by 5
            CompareValue=CompareValue-5;

            // Update pulse width of PWM and PRS_PWM
            PWM_WritePulseWidth(CompareValue);
            PRS_SEED_REG = CompareValue;
            
            // Wait till the key is released
            while(PRT0DR & 0x02);
        }
    }
}